=== modified file 'NEWS'
--- NEWS	2008-05-12 02:40:40 +0000
+++ NEWS	2008-05-12 03:04:40 +0000
@@ -22,6 +22,9 @@
 
   TESTING:
 
+    * New helper function for splitting test suites
+      ``split_suite_by_condition``. (Robert Collins)
+
   INTERNALS:
 
   API BREAKS:

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2008-05-12 02:40:40 +0000
+++ bzrlib/tests/__init__.py	2008-05-12 03:04:40 +0000
@@ -2358,6 +2358,28 @@
     return TestUtil.TestSuite(tests)
 
 
+def split_suite_by_condition(suite, condition):
+    """Split a test suite into two by a condition.
+    
+    :param suite: The suite to split.
+    :param condition: The condition to match on. Tests that match this
+        condition are returned in the first test suite, ones that do not match
+        are in the second suite.
+    :return: A tuple of two test suites, where the first contains tests from
+        suite matching the condition, and the second contains the remainder
+        from suite. The order within each output suite is the same as it was in
+        suite.
+    """ 
+    matched = []
+    did_not_match = []
+    for test in iter_suite_tests(suite):
+        if condition(test):
+            matched.append(test)
+        else:
+            did_not_match.append(test)
+    return TestUtil.TestSuite(matched), TestUtil.TestSuite(did_not_match)
+
+
 def split_suite_by_re(suite, pattern):
     """Split a test suite into two by a regular expression.
     
@@ -2370,16 +2392,7 @@
         suite. The order within each output suite is the same as it was in
         suite.
     """ 
-    matched = []
-    did_not_match = []
-    filter_re = re.compile(pattern)
-    for test in iter_suite_tests(suite):
-        test_id = test.id()
-        if filter_re.search(test_id):
-            matched.append(test)
-        else:
-            did_not_match.append(test)
-    return TestUtil.TestSuite(matched), TestUtil.TestSuite(did_not_match)
+    return split_suite_by_condition(suite, condition_id_re(pattern))
 
 
 def run_suite(suite, name='test', verbose=False, pattern=".*",

=== modified file 'bzrlib/tests/test_selftest.py'
--- bzrlib/tests/test_selftest.py	2008-05-08 21:22:06 +0000
+++ bzrlib/tests/test_selftest.py	2008-05-12 03:04:40 +0000
@@ -65,6 +65,7 @@
                           iter_suite_tests,
                           preserve_input,
                           randomize_suite,
+                          split_suite_by_condition,
                           split_suite_by_re,
                           test_lsprof,
                           test_suite,
@@ -1823,6 +1824,18 @@
         # necessarily.)
         self.assertEqual(len(self.all_names), len(_test_ids(randomized_suite)))
 
+    def test_split_suit_by_condition(self):
+        self.all_names = _test_ids(self.suite)
+        condition = condition_id_re('test_filter_suite_by_r')
+        split_suite = split_suite_by_condition(self.suite, condition)
+        filtered_name = ('bzrlib.tests.test_selftest.TestSelftestFiltering.'
+            'test_filter_suite_by_re')
+        self.assertEqual([filtered_name], _test_ids(split_suite[0]))
+        self.assertFalse(filtered_name in _test_ids(split_suite[1]))
+        remaining_names = list(self.all_names)
+        remaining_names.remove(filtered_name)
+        self.assertEqual(remaining_names, _test_ids(split_suite[1]))
+
     def test_split_suit_by_re(self):
         self.all_names = _test_ids(self.suite)
         split_suite = split_suite_by_re(self.suite, 'test_filter_suite_by_r')

